# Copyright 2005 James Bunton <james@delx.cjb.net>
# Licensed for distribution under the GPL version 2, check COPYING for details


import epollc
import os
import errno

EPOLLIN = 0x001
EPOLLPRI = 0x002
EPOLLOUT = 0x004
EPOLLRDNORM = 0x040
EPOLLRDBAND = 0x080
EPOLLWRNORM = 0x100
EPOLLWRBAND = 0x200
EPOLLMSG = 0x400
EPOLLERR = 0x008
EPOLLHUP = 0x010
EPOLLONESHOT = 1L << 30L
EPOLLET = 1L << 31L




class ePollError(Exception):
	def __init__(self, value):
		self.value = value
	
	def __str__(self):
		return repr(self.value)


class ePoller:
	EPOLL_CTL_ADD = 1
	EPOLL_CTL_DEL = 2
	EPOLL_CTL_MOD = 3

	def __init__(self, size=1024):
		ret = epollc.epoll_create(size)
		if(type(ret) == int):
			self.epfd = ret
		else:
			raise ePollError(ret[1])
	
	def __del__(self):
		self.close()
	
	def register(self, fd, eventMask):
		if(hasattr(fd, "fileno")):
			fd = fd.fileno()
		ret = epollc.epoll_ctl(self.epfd, self.EPOLL_CTL_ADD, fd, eventMask)
		if(ret and ret[1] != errno.EBADF):
			raise ePollError(ret[1])
	
	def unregister(self, fd):
		if(hasattr(fd, "fileno")):
			fd = fd.fileno()
		ret = epollc.epoll_ctl(self.epfd, self.EPOLL_CTL_DEL, fd, 0)
		if(ret and ret[1] != errno.ENOENT and ret[1] != errno.EBADF):
			raise ePollError(ret[1])
	
	def poll(self, timeout, maxevents=128):
		ret = epollc.epoll_wait(self.epfd, maxevents, timeout)
		if(type(ret) == list):
			return ret
		else:
			raise ePollError(ret[1])
	
	def close(self):
		f = os.fdopen(self.epfd)
		f.close()


